==> spirit_hub-1.0.0.11.tar.gz: spirit_hub-1.0.0.11/spec/models/tagset_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_phase_title' do
    let!(:operation) { create(:operation) }
    let!(:phase1) { create(:phase, title: 'Phase 1', operation:) }
    let!(:phase2) { create(:phase, title: 'Phase 2', operation:) }
    let!(:tagset) { create(:tagset, phase_id: phase1.id) }

    it 'default' do
      expect(tagset.phase_title).to eq('Phase 1')
    end

    it 'updates phase title' do
      tagset.update(phase_id: phase2.id)
      expect(tagset.reload.phase_title).to eq('Phase 2')
      phase2.update(title: 'Phase Changed')
      expect(tagset.reload.phase_title).to eq('Phase Changed')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }
    let!(:user3) { create(:user, first_name: 'User', last_name: '3') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed).to eq([])
      expect(tagset.informed_main).to eq([])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
    end

    it 'add user' do
      tagset.update!(informed_main: [user1.id], informed_cc: [user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id])
      expect(tagset.informed_cc).to eq([user2.id])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])
    end

    it 'add users and remove them' do
      tagset.update(informed_main: [user1.id, user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])

      tagset.update(informed_bcc: [])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user1.id, user2.id])

      tagset.update(informed: [user3.id])
      expect(tagset.reload.informed_main).to eq([user3.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user3.id])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'update_docs_fields' do
    let!(:tagset) { create(:tagset) }

    it 'without docs' do
      expect(tagset.reload.file_ext).to be_nil
      expect(tagset.reload.doc_file_size).to eq(nil)
    end

    it 'with one doc' do
      doc = create(:doc, file_ext: 'jpeg', file_size: 1000)
      tagset.update(doc_ids: [doc.id])
      expect(tagset.reload.file_ext).to eq('jpeg')
      expect(tagset.reload.doc_file_size).to eq(1000)
    end

    it 'with multiple docs' do
      doc1 = create(:doc, file_ext: 'jpeg', file_size: 1000)
      doc2 = create(:doc, file_ext: 'pdf', file_size: 2000)
      tagset.update(doc_ids: [doc1.id, doc2.id])
      expect(tagset.reload.file_ext).to eq('multiple')
      expect(tagset.reload.doc_file_size).to eq(3000)
    end
  end

  describe 'create_tagset_doc_from_file' do
    let(:user) { create(:user) }
    let(:file1) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 1'),
        filename: 'document1.pdf',
        type: 'application/pdf'
      )
    end

    let(:file2) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 2'),
        filename: 'document2.pdf',
        type: 'application/pdf'
      )
    end

    let(:files) { [file1] }

    subject { Tagset.create_tagset_doc_from_file(files, user) }

    context 'with multiple files' do
      let(:files) { [file1, file2] }
      it 'creates docs and a tagset with correct attributes' do
        files = [file1, file2]
        tagset = subject

        expect(tagset).to be_persisted
        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(files.size)

        tagset.doc_ids.each_with_index do |doc_id, index|
          doc = Doc.find(doc_id)
          file = files[index]

          expect(doc.responsible).to eq(user.id)
          expect(doc.title).to eq(file.original_filename)
          expect(doc.type).to eq('file')
        end
      end
    end

    context 'with a single file' do
      it 'creates a doc and a tagset with correct attributes' do
        tagset = subject

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file1.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    context 'with files as file_paths' do
      let(:files) { [Rails.root.join('packages/spirit_hub/spec/fixtures/files/one_page.pdf')] }

      it 'creates a doc and a tagset with correct attributes' do
        tagset = subject

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq('one_page.pdf')
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq('one_page.pdf')
        expect(doc.type).to eq('file')
      end
    end

    it 'add a journal entry' do
      allow(SpiritHub::JournalEntry).to receive(:create)
      tagset = subject

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_CREATED',
        tagset_id: tagset.id,
        user_id: user.id,
        action: "Document #{file1.original_filename} créé par #{user.full_name}",
        action_msg: nil,
        opts: instance_of(Hash)
      ).once
    end
  end

  describe 'add_tagset_doc_to_object_by_id' do
    let!(:user) { create(:user) }
    let!(:model) { create(:tagset_doc, title: 'My doc title') }
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let(:tagset) { create(:tagset, operation_id: operation.id) }
    let(:completeness) { create(:completeness, title: 'Completness', tagset_id: tagset.id) }
    let(:notifier) { instance_double(SpiritHub::TagsetNotifier) }

    before do
      allow(SpiritHub::TagsetNotifier).to receive(:new).with(have_attributes(title: 'My doc title'), user).and_return(notifier)
      allow(notifier).to receive(:call)
    end

    context 'when a Tagset' do
      let(:id_param) { tagset.id }

      it 'add the model to the tagset' do
        allow(SpiritHub::JournalEntry).to receive(:create)

        model.add_tagset_doc_to_object_by_id(id_param, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(tagset.reload.tagset_ids).to include(model.id)

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DOC_CLASSIFIED',
          tagset_id: model.id,
          user_id: user.id,
          action: "Document #{model.title} classé dans #{tagset.title}",
          action_msg: nil,
          opts: instance_of(Hash)
        ).once

        expect(notifier).to have_received(:call).once
      end
    end

    context 'when a Completeness' do
      let(:id_param) { "completeness_#{completeness.id}" }

      it 'add the model to the completeness' do
        allow(SpiritHub::JournalEntry).to receive(:create)

        model.add_tagset_doc_to_object_by_id(id_param, user)
        model.add_tagset_doc_to_object_by_id(id_param, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.title).to eq(completeness.title)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(completeness.reload.object_ids).to include(model.id)
        expect(tagset.reload.tagset_ids).to include(model.id.to_s)
      end

      it 'add a log' do
        allow(SpiritHub::JournalEntry).to receive(:create)
        model.add_tagset_doc_to_object_by_id(id_param, user)

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DOC_ADDED_TO_COMPLETENESS',
          tagset_id: model.id,
          user_id: user.id,
          action: "Complétude des documents pour #{completeness.title} par #{user.full_name}",
          action_msg: nil,
          opts: instance_of(Hash)
        ).once
      end
    end

    context 'when has a completeness and a tagset' do
      let(:id_param) { "completeness_#{completeness.id}" }
      let!(:other_tagset) { create(:tagset, title: 'Other Tagset', operation_id: operation.id) }
      let!(:other_completeness) { create(:completeness, title: 'Other Completness', tagset_id: other_tagset.id) }

      before do
        model.add_tagset_doc_to_object_by_id("completeness_#{other_completeness.id}", user)
      end

      it 'remove the previous completeness and tagset' do
        expect(other_completeness.reload.object_ids.to_a).to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).to include(model.id.to_s)

        model.add_tagset_doc_to_object_by_id(id_param, user)

        expect(completeness.reload.object_ids.to_a).to include(model.id)
        expect(tagset.reload.tagset_ids.to_a).to include(model.id.to_s)
        expect(other_completeness.reload.object_ids.to_a).not_to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).not_to include(model.id.to_s)
      end
    end
  end

  describe 'add_doc_to_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }
    let(:file1) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 1'),
        filename: 'document1.pdf',
        type: 'application/pdf'
      )
    end

    let(:file2) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 2'),
        filename: 'document2.pdf',
        type: 'application/pdf'
      )
    end

    it 'with multiple files' do
      files = [file1, file2]
      tagset_doc.add_doc_to_tagset_doc(files, user)

      expect(tagset_doc.doc_ids.size).to eq(files.size)

      tagset_doc.doc_ids.each_with_index do |doc_id, index|
        doc = Doc.find(doc_id)
        file = files[index]

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    it 'with a single file' do
      tagset_doc.add_doc_to_tagset_doc(file1, user)
      expect(tagset_doc.doc_ids.size).to eq(1)

      doc = Doc.find(tagset_doc.doc_ids.first)

      expect(doc.responsible).to eq(user.id)
      expect(doc.title).to eq(file1.original_filename)
      expect(doc.type).to eq('file')
    end
  end

  describe 'remove_doc_from_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file) }

    it 'works' do
      doc = Doc.find(tagset_doc.doc_ids.first)
      tagset_doc.remove_doc_from_tagset_doc(doc, user)

      expect(tagset_doc.doc_ids).to eq([])
      expect(Doc.find_by(id: doc.id)).to be_nil
    end
  end

  describe 'update_phase_percent' do
    let!(:operation) { create(:operation) }
    let!(:phase) { create(:phase, title: 'Phase 1', operation:) }
    let!(:tagset_directory1) { create(:tagset_directory, phase_id: phase.id) }
    let!(:completeness1) { create(:completeness, tagset_id: tagset_directory1.id) }
    let!(:tagset_directory2) { create(:tagset_directory, phase_id: phase.id) }
    let!(:completeness2) { create(:completeness, tagset_id: tagset_directory2.id) }
    let!(:completeness3) { create(:completeness, tagset_id: tagset_directory2.id) }

    it 'default' do
      expect(phase.reload.percentage).to eq(0)
    end

    it 'updates phase percentage when completeness added' do
      tagset_doc = create(:tagset_doc)
      completeness1.update(object_ids: [tagset_doc.id])
      expect(phase.reload.percentage).to eq(33)
    end

    it 'updates phase percentage when completeness removed' do
      tagset_doc = create(:tagset_doc)
      completeness2.update(object_ids: [tagset_doc.id])
      expect(phase.reload.percentage).to eq(33)
      completeness2.update(object_ids: [])
      expect(phase.reload.percentage).to eq(0)
    end
  end

  describe 'versioned docs' do
    let(:tagset) { create(:tagset_doc) }
    let(:doc) { create :doc }
    let(:version) { create(:doc, version_of: doc.id.to_s) }

    before do
      tagset.update(doc_ids: [doc.id, version.id])
    end

    it { expect(tagset).to be_one_doc }
    it { expect(tagset.all_docs.count).to eq 2 }
    it { expect(tagset.docs.count).to eq 1 }
    it { expect(tagset.doc).to eq doc }
  end
end

==> spirit_hub-1.0.0.15.tar.gz: spirit_hub-1.0.0.15/spec/models/tagset_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_phase_title' do
    let!(:operation) { create(:operation) }
    let!(:phase1) { create(:phase, title: 'Phase 1', operation:) }
    let!(:phase2) { create(:phase, title: 'Phase 2', operation:) }
    let!(:tagset) { create(:tagset, phase_id: phase1.id) }

    it 'default' do
      expect(tagset.phase_title).to eq('Phase 1')
    end

    it 'updates phase title' do
      tagset.update(phase_id: phase2.id)
      expect(tagset.reload.phase_title).to eq('Phase 2')
      phase2.update(title: 'Phase Changed')
      expect(tagset.reload.phase_title).to eq('Phase Changed')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }
    let!(:user3) { create(:user, first_name: 'User', last_name: '3') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed).to eq([])
      expect(tagset.informed_main).to eq([])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
    end

    it 'add user' do
      tagset.update!(informed_main: [user1.id], informed_cc: [user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id])
      expect(tagset.informed_cc).to eq([user2.id])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])
    end

    it 'add users and remove them' do
      tagset.update(informed_main: [user1.id, user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])

      tagset.update(informed_bcc: [])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user1.id, user2.id])

      tagset.update(informed: [user3.id])
      expect(tagset.reload.informed_main).to eq([user3.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user3.id])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'update_docs_fields' do
    let!(:tagset) { create(:tagset) }

    it 'without docs' do
      expect(tagset.reload.file_ext).to be_nil
      expect(tagset.reload.doc_file_size).to eq(nil)
    end

    it 'with one doc' do
      doc = create(:doc, file_ext: 'jpeg', file_size: 1000)
      tagset.update(doc_ids: [doc.id])
      expect(tagset.reload.file_ext).to eq('jpeg')
      expect(tagset.reload.doc_file_size).to eq(1000)
    end

    it 'with multiple docs' do
      doc1 = create(:doc, file_ext: 'jpeg', file_size: 1000)
      doc2 = create(:doc, file_ext: 'pdf', file_size: 2000)
      tagset.update(doc_ids: [doc1.id, doc2.id])
      expect(tagset.reload.file_ext).to eq('multiple')
      expect(tagset.reload.doc_file_size).to eq(3000)
    end
  end

  describe 'create_tagset_doc_from_file' do
    let(:user) { create(:user) }
    let(:file1) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 1'),
        filename: 'document1.pdf',
        type: 'application/pdf'
      )
    end

    let(:file2) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 2'),
        filename: 'document2.pdf',
        type: 'application/pdf'
      )
    end

    let(:files) { [file1] }

    subject { Tagset.create_tagset_doc_from_file(files, user) }

    context 'with multiple files' do
      let(:files) { [file1, file2] }
      it 'creates docs and a tagset with correct attributes' do
        files = [file1, file2]
        tagset = subject

        expect(tagset).to be_persisted
        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(files.size)

        tagset.doc_ids.each_with_index do |doc_id, index|
          doc = Doc.find(doc_id)
          file = files[index]

          expect(doc.responsible).to eq(user.id)
          expect(doc.title).to eq(file.original_filename)
          expect(doc.type).to eq('file')
        end
      end
    end

    context 'with a single file' do
      it 'creates a doc and a tagset with correct attributes' do
        tagset = subject

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file1.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    context 'with files as file_paths' do
      let(:files) { [Rails.root.join('packages/spirit_hub/spec/fixtures/files/one_page.pdf')] }

      it 'creates a doc and a tagset with correct attributes' do
        tagset = subject

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq('one_page.pdf')
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq('one_page.pdf')
        expect(doc.type).to eq('file')
      end
    end

    it 'add a journal entry' do
      allow(SpiritHub::JournalEntry).to receive(:create)
      tagset = subject

      expect(SpiritHub::JournalEntry).to have_received(:create).with(
        code: 'DOC_CREATED',
        tagset_id: tagset.id,
        user_id: user.id,
        action: "Document #{file1.original_filename} créé par #{user.full_name}",
        action_msg: nil,
        opts: instance_of(Hash)
      ).once
    end
  end

  describe 'add_tagset_doc_to_object_by_id' do
    let!(:user) { create(:user) }
    let!(:reader) { create(:user) }
    let!(:writer) { create(:user) }
    let!(:model) { create(:tagset_doc, title: 'My doc title') }
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let(:tagset) { create(:tagset, operation_id: operation.id, readers: [reader.id.to_s], writers: [writer.id.to_s]) }
    let(:completeness) { create(:completeness, title: 'Completness', tagset_id: tagset.id) }
    let(:notifier) { instance_double(SpiritHub::TagsetNotifier) }

    before do
      allow(SpiritHub::TagsetNotifier).to receive(:new).with(model, user).and_return(notifier)
      allow(notifier).to receive(:call)
    end

    context 'when a Tagset' do
      let(:id_param) { tagset.id }

      it 'add the model to the tagset' do
        allow(SpiritHub::JournalEntry).to receive(:create)

        model.add_tagset_doc_to_object_by_id(id_param, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(model.readers).to include(reader.id.to_s)
        expect(model.writers).to include(writer.id.to_s)
        expect(tagset.reload.tagset_ids).to include(model.id)

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DOC_CLASSIFIED',
          tagset_id: model.id,
          user_id: user.id,
          action: "Document #{model.title} classé dans #{tagset.title}",
          action_msg: nil,
          opts: instance_of(Hash)
        ).once

        expect(notifier).to have_received(:call).once
      end
    end

    context 'when a Completeness' do
      let(:id_param) { "completeness_#{completeness.id}" }

      it 'add the model to the completeness' do
        allow(SpiritHub::JournalEntry).to receive(:create)

        model.add_tagset_doc_to_object_by_id(id_param, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.title).to eq(completeness.title)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(model.readers).to include(reader.id.to_s)
        expect(model.writers).to include(writer.id.to_s)
        expect(completeness.reload.object_ids).to include(model.id)
        expect(tagset.reload.tagset_ids).to include(model.id.to_s)
      end

      it 'add a log' do
        allow(SpiritHub::JournalEntry).to receive(:create)
        model.add_tagset_doc_to_object_by_id(id_param, user)

        expect(SpiritHub::JournalEntry).to have_received(:create).with(
          code: 'DOC_ADDED_TO_COMPLETENESS',
          tagset_id: model.id,
          user_id: user.id,
          action: "Complétude des documents pour #{completeness.title} par #{user.full_name}",
          action_msg: nil,
          opts: instance_of(Hash)
        ).once
      end
    end

    context 'when has a completeness and a tagset' do
      let(:id_param) { "completeness_#{completeness.id}" }
      let!(:other_tagset) { create(:tagset, title: 'Other Tagset', operation_id: operation.id) }
      let!(:other_completeness) { create(:completeness, title: 'Other Completness', tagset_id: other_tagset.id) }

      before do
        model.add_tagset_doc_to_object_by_id("completeness_#{other_completeness.id}", user)
      end

      it 'remove the previous completeness and tagset' do
        expect(other_completeness.reload.object_ids.to_a).to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).to include(model.id.to_s)

        model.add_tagset_doc_to_object_by_id(id_param, user)

        expect(completeness.reload.object_ids.to_a).to include(model.id)
        expect(tagset.reload.tagset_ids.to_a).to include(model.id.to_s)
        expect(other_completeness.reload.object_ids.to_a).not_to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).not_to include(model.id.to_s)
      end
    end
  end

  describe 'add_doc_to_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }
    let(:file1) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 1'),
        filename: 'document1.pdf',
        type: 'application/pdf'
      )
    end

    let(:file2) do
      ActionDispatch::Http::UploadedFile.new(
        tempfile: StringIO.new('File content 2'),
        filename: 'document2.pdf',
        type: 'application/pdf'
      )
    end

    it 'with multiple files' do
      files = [file1, file2]
      tagset_doc.add_doc_to_tagset_doc(files, user)

      expect(tagset_doc.doc_ids.size).to eq(files.size)

      tagset_doc.doc_ids.each_with_index do |doc_id, index|
        doc = Doc.find(doc_id)
        file = files[index]

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    it 'with a single file' do
      tagset_doc.add_doc_to_tagset_doc(file1, user)
      expect(tagset_doc.doc_ids.size).to eq(1)

      doc = Doc.find(tagset_doc.doc_ids.first)

      expect(doc.responsible).to eq(user.id)
      expect(doc.title).to eq(file1.original_filename)
      expect(doc.type).to eq('file')
    end
  end

  describe 'remove_doc_from_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file) }

    it 'works' do
      doc = Doc.find(tagset_doc.doc_ids.first)
      tagset_doc.remove_doc_from_tagset_doc(doc, user)

      expect(tagset_doc.doc_ids).to eq([])
      expect(Doc.find_by(id: doc.id)).to be_nil
    end
  end

  describe 'update_phase_percent' do
    let!(:operation) { create(:operation) }
    let!(:phase) { create(:phase, title: 'Phase 1', operation:) }
    let!(:tagset_directory1) { create(:tagset_directory, phase_id: phase.id) }
    let!(:completeness1) { create(:completeness, tagset_id: tagset_directory1.id) }
    let!(:tagset_directory2) { create(:tagset_directory, phase_id: phase.id) }
    let!(:completeness2) { create(:completeness, tagset_id: tagset_directory2.id) }
    let!(:completeness3) { create(:completeness, tagset_id: tagset_directory2.id) }

    it 'default' do
      expect(phase.reload.percentage).to eq(0)
    end

    it 'updates phase percentage when completeness added' do
      tagset_doc = create(:tagset_doc)
      completeness1.update(object_ids: [tagset_doc.id])
      expect(phase.reload.percentage).to eq(33)
    end

    it 'updates phase percentage when completeness removed' do
      tagset_doc = create(:tagset_doc)
      completeness2.update(object_ids: [tagset_doc.id])
      expect(phase.reload.percentage).to eq(33)
      completeness2.update(object_ids: [])
      expect(phase.reload.percentage).to eq(0)
    end
  end

  describe 'versioned docs' do
    let(:tagset) { create(:tagset_doc) }
    let(:doc) { create :doc }
    let(:version) { create(:doc, version_of: doc.id.to_s) }

    before do
      tagset.update(doc_ids: [doc.id, version.id])
    end

    it { expect(tagset).to be_one_doc }
    it { expect(tagset.all_docs.count).to eq 2 }
    it { expect(tagset.docs.count).to eq 1 }
    it { expect(tagset.doc).to eq doc }
  end
end
